package NewsFeedPublisher.data;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.sql.DataSource;

public class DataAccessObject {
	
	private static DataSource dataSource;
	private static Object idLock = new Object();
	
	public static void setDataSource(DataSource dataSource) {
	
		DataAccessObject.dataSource = dataSource;
	}
	
	protected static Connection getConnection() {
		try {
			return dataSource.getConnection();
		} catch (SQLException sqle) {
			throw new RuntimeException(sqle);
		}
	}
	
	protected static void close(Statement statement, Connection connection) {
		
		close(null, statement, connection);
	}
	
	protected static void close(ResultSet rs, Statement statement, Connection connection){
		
		try {		
	        if (rs != null)
	        	rs.close();
	        if (statement != null)
	            statement.close();
	        if (connection != null)
	            connection.close();
	      } catch (SQLException sqle) {
	         throw new RuntimeException(sqle);
	      }
	}
	
	protected static Long getUniqueId() {
		
		ResultSet rs = null;
		PreparedStatement statement = null;
		Connection connection = null;
		
		try {
			connection = getConnection();
			
			//allow only a single thread to enter the code block
			synchronized(idLock){
				statement = connection.prepareStatement("select next_value from sequence");
				rs = statement.executeQuery();
				
				//Moves the cursor to the first row in this ResultSet object
				rs.first();
				long id = rs.getLong(1);
				statement.close();
				statement = connection.prepareStatement("update sequence set next_value = ?");
				statement.setLong(1, id + 1);
				statement.executeUpdate();
				statement.close();
				return new Long(id);
			}
		} catch (SQLException sqle) {
			throw new RuntimeException(sqle);
		} finally {
			close(rs,statement, connection);
		}
	}

}
