package NewsFeedPublisher.web;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import NewsFeedPublisher.data.User;
import NewsFeedPublisher.data.UserDAO;

public class EditUserServlet extends HttpServlet {
	
	private Logger logger = Logger.getLogger(this.getClass());
	private RequestDispatcher jsp;
	   
	public void init(ServletConfig config) throws ServletException {
	
		ServletContext context = config.getServletContext();
	    jsp = context.getRequestDispatcher("/WEB-INF/jsp/edit-user.jsp");
	}
	
	protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		
		logger.debug("doGet()");
	    String idString = req.getParameter("id");
	    Long id = new Long(idString);
	    User user = new UserDAO().find(id);
	    req.setAttribute("user", user);
	    jsp.forward(req, resp);
	}
	
	protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		
		String id = req.getParameter("id");
		
		//check if cancel button was pressed
		String cancelButton = req.getParameter("cancel-button");
		if(cancelButton != null) {
			
			logger.debug("cancel button pressed");
			resp.sendRedirect("view-user?id=" + id);
			return;
		}
		Map<String, String> errors = validate(req);
		if(!errors.isEmpty()) {
			
			logger.debug("validation error");
			jsp.forward(req, resp);
			return;
		}
		User user = (User) req.getAttribute("user");
		new UserDAO().update(user);
		resp.sendRedirect("view-user?id=" + id);
	}
	
	public static Map<String, String> validate(HttpServletRequest req) {
		
		  User user = new User();
	      HashMap<String, String> errors = new HashMap<String, String>();
	      req.setAttribute("errors", errors);
	      req.setAttribute("user", user);

	      String idString = req.getParameter("id");
	      if (idString != null && idString.length() > 0) {    	  
	          Long id = new Long(idString);
	          user.setId(id);
	      }
	      
	      String username = req.getParameter("username");
	      if (username == null || username.trim().length() == 0)
	         errors.put("username", "Username required.");
	      user.setUsername(username);

	      String password = req.getParameter("password");
	      if (password == null || password.trim().length() == 0)
	         errors.put("password", "Password required.");
	      user.setPassword(password);
	      
	      return errors;
	}
}