package NewsFeedPublisher.ws;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.OutputStream;
import java.io.StringWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import NewsFeedPublisher.data.NewsItem;
import NewsFeedPublisher.data.NewsItemDAO;

public class PublishNewsItemService extends HttpServlet {
	
	private Logger logger = Logger.getLogger(this.getClass());
	
	protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		
		logger.debug("doPost for Publish web service");
		
		//read the xml request document
		//provide access to the character data coming in from the client
		BufferedReader br =req.getReader();
		
		SAXBuilder builder = new SAXBuilder();
		Document requestDocument = null;
		try {
			//create an object representation of the XML document and returns a reference to this object
			//extracts the data from the buffered reader passed into it and constructs the document 
			requestDocument = builder.build(br);
		} catch(JDOMException jdome) {
			throw new RuntimeException(jdome);
		}
		
		//extract title and link from the request
		Element item = requestDocument.getRootElement();
		Element titleElement = item.getChild("title");
		String title = titleElement.getText();
		Element linkElement = item.getChild("link");
		String link = linkElement.getText();
		
		//create a news item from the submitted data
		NewsItem newsItem = new NewsItem();
		newsItem.setTitle(title);
		newsItem.setUrl(link);
		new NewsItemDAO().create(newsItem);
		
		//create response document with id of newly created news item
		Element idElement = new Element("id");
		idElement.addContent(newsItem.getId().toString());
		Document responseDocument = new Document(idElement);
		StringWriter sw = new StringWriter();
		//serialize the XML document into a string that will be returned to the client
		XMLOutputter outputter = new XMLOutputter();
		//print the document to the writer
		outputter.output(responseDocument, sw);
		String responseDocumentString = sw.toString();
		
		//return the response document to the client
		//create byte array containing the raw binary representation of the message data
		byte[] responseBytes = responseDocumentString.getBytes("UTF-8");
		resp.setContentLength(responseBytes.length);
		resp.setContentType("text/xml");
		OutputStream os = resp.getOutputStream();
		os.write(responseBytes);
		os.flush();
	}
}